@@ -18,27 +18,21 @@ use librespot_core::cache::Cache;
 use librespot_playback::audio_backend;
 use log::{error, info, trace};
 
-mod album;
-mod artist;
 mod authentication;
 mod command;
 mod commands;
 mod config;
-mod episode;
 mod events;
 mod library;
-mod playable;
-mod playlist;
+mod model;
 mod queue;
 mod serialization;
 mod sharing;
-mod show;
 mod spotify;
 mod spotify_api;
 mod spotify_url;
 mod spotify_worker;
 mod theme;
-mod track;
 mod traits;
 mod ui;
 mod utils;
