@@ -1,28 +1,25 @@
-use crate::album::Album;
-use crate::artist::Artist;
-use crate::episode::Episode;
-use crate::playable::Playable;
-use crate::playlist::Playlist;
+use crate::model::album::Album;
+use crate::model::artist::Artist;
+use crate::model::episode::Episode;
+use crate::model::playable::Playable;
+use crate::model::playlist::Playlist;
+use crate::model::track::Track;
 use crate::spotify_worker::WorkerCommand;
-use crate::track::Track;
 use crate::ui::pagination::{ApiPage, ApiResult};
 use chrono::{DateTime, Duration as ChronoDuration, Utc};
-use failure::Error;
 use futures::channel::oneshot;
 use log::{debug, error, info};
-use rspotify::blocking::client::ApiError;
-use rspotify::blocking::client::Spotify as SpotifyAPI;
-use rspotify::model::album::{FullAlbum, SavedAlbum};
-use rspotify::model::artist::FullArtist;
-use rspotify::model::page::{CursorBasedPage, Page};
-use rspotify::model::playlist::FullPlaylist;
-use rspotify::model::recommend::Recommendations;
-use rspotify::model::search::SearchResult;
-use rspotify::model::show::{FullEpisode, FullShow, Show};
-use rspotify::model::track::{FullTrack, SavedTrack, SimplifiedTrack};
-use rspotify::model::user::PrivateUser;
-use rspotify::senum::{AlbumType, Country, SearchType};
-use serde_json::{json, Map};
+
+use rspotify::http::HttpError;
+use rspotify::model::{
+    AlbumId, AlbumType, ArtistId, CursorBasedPage, EpisodeId, FullAlbum, FullArtist, FullEpisode,
+    FullPlaylist, FullShow, FullTrack, ItemPositions, Market, Page, PlayableId, PlaylistId,
+    PrivateUser, Recommendations, SavedAlbum, SavedTrack, SearchResult, SearchType, Show, ShowId,
+    SimplifiedTrack, TrackId, UserId,
+};
+use rspotify::{prelude::*, AuthCodeSpotify, ClientError, ClientResult, Token};
+use std::collections::HashSet;
+use std::iter::FromIterator;
 use std::sync::{Arc, RwLock};
 use std::thread;
 use std::time::Duration;
@@ -30,9 +27,8 @@ use tokio::sync::mpsc;
 
 #[derive(Clone)]
 pub struct WebApi {
-    api: Arc<RwLock<SpotifyAPI>>,
+    api: AuthCodeSpotify,
     user: Option<String>,
-    country: Option<Country>,
     worker_channel: Arc<RwLock<Option<mpsc::UnboundedSender<WorkerCommand>>>>,
     token_expiration: Arc<RwLock<DateTime<Utc>>>,
 }
@@ -40,9 +36,8 @@ pub struct WebApi {
 impl WebApi {
     pub fn new() -> WebApi {
         WebApi {
-            api: Arc::new(RwLock::new(SpotifyAPI::default())),
+            api: AuthCodeSpotify::default(),
             user: None,
-            country: None,
             worker_channel: Arc::new(RwLock::new(None)),
             token_expiration: Arc::new(RwLock::new(Utc::now())),
         }
@@ -52,10 +47,6 @@ impl WebApi {
         self.user = user;
     }
 
-    pub fn set_country(&mut self, country: Option<Country>) {
-        self.country = country;
-    }
-
     pub(crate) fn set_worker_channel(
         &mut self,
         channel: Arc<RwLock<Option<mpsc::UnboundedSender<WorkerCommand>>>>,
@@ -87,8 +78,13 @@ impl WebApi {
         {
             channel.send(cmd).expect("can't send message to worker");
             let token = futures::executor::block_on(token_rx).unwrap();
-            self.api.write().expect("can't writelock api").access_token =
-                Some(token.access_token.to_string());
+            *self.api.token.lock().expect("can't writelock api token") = Some(Token {
+                access_token: token.access_token,
+                expires_in: chrono::Duration::seconds(token.expires_in.into()),
+                scopes: HashSet::from_iter(token.scope),
+                expires_at: None,
+                refresh_token: None,
+            });
             *self
                 .token_expiration
                 .write()
@@ -102,32 +98,32 @@ impl WebApi {
     /// retries once when rate limits are hit
     fn api_with_retry<F, R>(&self, cb: F) -> Option<R>
     where
-        F: Fn(&SpotifyAPI) -> Result<R, Error>,
+        F: Fn(&AuthCodeSpotify) -> ClientResult<R>,
     {
-        let result = {
-            let api = self.api.read().expect("can't read api");
-            cb(&api)
-        };
+        let result = { cb(&self.api) };
         match result {
             Ok(v) => Some(v),
-            Err(e) => {
-                debug!("api error: {:?}", e);
-                if let Ok(apierror) = e.downcast::<ApiError>() {
-                    match apierror {
-                        ApiError::RateLimited(d) => {
-                            debug!("rate limit hit. waiting {:?} seconds", d);
-                            thread::sleep(Duration::from_secs(d.unwrap_or(0) as u64));
-                            let api = self.api.read().expect("can't read api");
-                            cb(&api).ok()
+            Err(ClientError::Http(error)) => {
+                debug!("http error: {:?}", error);
+                if let HttpError::StatusCode(response) = error.as_ref() {
+                    match response.status() {
+                        429 => {
+                            let waiting_duration = response
+                                .header("Retry-After")
+                                .and_then(|v| v.parse::<u64>().ok());
+                            debug!("rate limit hit. waiting {:?} seconds", waiting_duration);
+                            thread::sleep(
+                                Duration::from_secs(waiting_duration.unwrap_or(0) as u64),
+                            );
+                            cb(&self.api).ok()
                         }
-                        ApiError::Unauthorized => {
+                        401 => {
                             debug!("token unauthorized. trying refresh..");
                             self.update_token();
-                            let api = self.api.read().expect("can't read api");
-                            cb(&api).ok()
+                            cb(&self.api).ok()
                         }
-                        e => {
-                            error!("unhandled api error: {}", e);
+                        _ => {
+                            error!("unhandled api error: {:?}", response);
                             None
                         }
                     }
@@ -135,17 +131,34 @@ impl WebApi {
                     None
                 }
             }
+            Err(e) => {
+                error!("unhandled api error: {}", e);
+                None
+            }
         }
     }
 
     pub fn append_tracks(
         &self,
         playlist_id: &str,
-        tracks: &[String],
+        tracks: &[Playable],
         position: Option<i32>,
     ) -> bool {
         self.api_with_retry(|api| {
-            api.user_playlist_add_tracks(self.user.as_ref().unwrap(), playlist_id, tracks, position)
+            let trackids: Vec<Box<dyn PlayableId>> = tracks
+                .iter()
+                .map(|playable| {
+                    Box::new(
+                        TrackId::from_id(playable.id().as_ref().unwrap_or(&"".to_string()))
+                            .unwrap(),
+                    ) as Box<dyn PlayableId>
+                })
+                .collect();
+            api.playlist_add_items(
+                &PlaylistId::from_id(playlist_id).unwrap(),
+                trackids.iter().map(|id| id.as_ref()),
+                position,
+            )
         })
         .is_some()
     }
@@ -154,31 +167,45 @@ impl WebApi {
         &self,
         playlist_id: &str,
         snapshot_id: &str,
-        track_pos_pairs: &[(&Track, usize)],
+        playables: &[Playable],
     ) -> bool {
-        let mut tracks = Vec::new();
-        for (track, pos) in track_pos_pairs {
-            let track_occurrence = json!({
-                "uri": format!("spotify:track:{}", track.id.clone().unwrap()),
-                "positions": [pos]
-            });
-            let track_occurrence_object = track_occurrence.as_object();
-            tracks.push(track_occurrence_object.unwrap().clone());
-        }
-        self.api_with_retry(|api| {
-            api.user_playlist_remove_specific_occurrenes_of_tracks(
-                self.user.as_ref().unwrap(),
-                playlist_id,
-                tracks.clone(),
-                Some(snapshot_id.to_string()),
+        self.api_with_retry(move |api| {
+            let playable_ids: Vec<Box<dyn PlayableId>> = playables
+                .iter()
+                .map(|playable| match playable {
+                    Playable::Track(track) => {
+                        Box::new(TrackId::from_id(&track.id.clone().unwrap_or_default()).unwrap())
+                            as Box<dyn PlayableId>
+                    }
+                    Playable::Episode(episode) => {
+                        Box::new(EpisodeId::from_id(&episode.id).unwrap()) as Box<dyn PlayableId>
+                    }
+                })
+                .collect();
+            let positions = playables
+                .iter()
+                .map(|playable| [playable.list_index() as u32])
+                .collect::<Vec<_>>();
+            let item_pos: Vec<ItemPositions> = playable_ids
+                .iter()
+                .zip(positions.iter())
+                .map(|(id, positions)| ItemPositions {
+                    id: id.as_ref(),
+                    positions,
+                })
+                .collect();
+            api.playlist_remove_specific_occurrences_of_items(
+                &PlaylistId::from_id(playlist_id).unwrap(),
+                item_pos,
+                Some(snapshot_id),
             )
         })
         .is_some()
     }
 
     pub fn overwrite_playlist(&self, id: &str, tracks: &[Playable]) {
-        // extract only track IDs
-        let mut tracks: Vec<String> = tracks.iter().filter_map(|track| track.id()).collect();
+        // create mutable copy for chunking
+        let mut tracks: Vec<Playable> = tracks.to_vec();
 
         // we can only send 100 tracks per request
         let mut remainder = if tracks.len() > 100 {
@@ -188,7 +215,22 @@ impl WebApi {
         };
 
         if let Some(()) = self.api_with_retry(|api| {
-            api.user_playlist_replace_tracks(self.user.as_ref().unwrap(), id, &tracks)
+            let playable_ids: Vec<Box<dyn PlayableId>> = tracks
+                .iter()
+                .map(|playable| match playable {
+                    Playable::Track(track) => {
+                        Box::new(TrackId::from_id(&track.id.clone().unwrap_or_default()).unwrap())
+                            as Box<dyn PlayableId>
+                    }
+                    Playable::Episode(episode) => {
+                        Box::new(EpisodeId::from_id(&episode.id).unwrap()) as Box<dyn PlayableId>
+                    }
+                })
+                .collect();
+            api.playlist_replace_items(
+                &PlaylistId::from_id(id).unwrap(),
+                playable_ids.iter().map(|p| p.as_ref()),
+            )
         }) {
             debug!("saved {} tracks to playlist {}", tracks.len(), id);
             while let Some(ref mut tracks) = remainder.clone() {
@@ -213,7 +255,7 @@ impl WebApi {
     }
 
     pub fn delete_playlist(&self, id: &str) -> bool {
-        self.api_with_retry(|api| api.user_playlist_unfollow(self.user.as_ref().unwrap(), id))
+        self.api_with_retry(|api| api.playlist_unfollow(&PlaylistId::from_id(id).unwrap()))
             .is_some()
     }
 
@@ -221,57 +263,83 @@ impl WebApi {
         &self,
         name: &str,
         public: Option<bool>,
-        description: Option<String>,
+        description: Option<&str>,
     ) -> Option<String> {
         let result = self.api_with_retry(|api| {
             api.user_playlist_create(
-                self.user.as_ref().unwrap(),
+                &UserId::from_id(self.user.as_ref().unwrap()).unwrap(),
                 name,
                 public,
-                description.clone(),
+                None,
+                description,
             )
         });
-        result.map(|r| r.id)
+        result.map(|r| r.id.id().to_string())
     }
 
     pub fn album(&self, album_id: &str) -> Option<FullAlbum> {
-        self.api_with_retry(|api| api.album(album_id))
+        self.api_with_retry(|api| api.album(&AlbumId::from_id(album_id).unwrap()))
     }
 
     pub fn artist(&self, artist_id: &str) -> Option<FullArtist> {
-        self.api_with_retry(|api| api.artist(artist_id))
+        self.api_with_retry(|api| api.artist(&ArtistId::from_id(artist_id).unwrap()))
     }
 
     pub fn playlist(&self, playlist_id: &str) -> Option<FullPlaylist> {
-        self.api_with_retry(|api| api.playlist(playlist_id, None, self.country))
+        self.api_with_retry(|api| {
+            api.playlist(
+                &PlaylistId::from_id(playlist_id).unwrap(),
+                None,
+                Some(&Market::FromToken),
+            )
+        })
     }
 
     pub fn track(&self, track_id: &str) -> Option<FullTrack> {
-        self.api_with_retry(|api| api.track(track_id))
+        self.api_with_retry(|api| api.track(&TrackId::from_id(track_id).unwrap()))
     }
 
     pub fn get_show(&self, show_id: &str) -> Option<FullShow> {
-        self.api_with_retry(|api| api.get_a_show(show_id.to_string(), self.country))
+        self.api_with_retry(|api| {
+            api.get_a_show(&ShowId::from_id(show_id).unwrap(), Some(&Market::FromToken))
+        })
     }
 
     pub fn episode(&self, episode_id: &str) -> Option<FullEpisode> {
-        self.api_with_retry(|api| api.get_an_episode(episode_id.to_string(), self.country))
+        self.api_with_retry(|api| {
+            api.get_an_episode(
+                &EpisodeId::from_id(episode_id).unwrap(),
+                Some(&Market::FromToken),
+            )
+        })
     }
 
     pub fn recommendations(
         &self,
-        seed_artists: Option<Vec<String>>,
-        seed_genres: Option<Vec<String>>,
-        seed_tracks: Option<Vec<String>>,
+        seed_artists: Option<Vec<&str>>,
+        seed_genres: Option<Vec<&str>>,
+        seed_tracks: Option<Vec<&str>>,
     ) -> Option<Recommendations> {
         self.api_with_retry(|api| {
+            let seed_artistids = seed_artists.as_ref().map(|artistids| {
+                artistids
+                    .iter()
+                    .map(|id| ArtistId::from_id(id).unwrap())
+                    .collect::<Vec<ArtistId>>()
+            });
+            let seed_trackids = seed_tracks.as_ref().map(|trackids| {
+                trackids
+                    .iter()
+                    .map(|id| TrackId::from_id(id).unwrap())
+                    .collect::<Vec<TrackId>>()
+            });
             api.recommendations(
-                seed_artists.clone(),
+                std::iter::empty(),
+                seed_artistids.as_ref(),
                 seed_genres.clone(),
-                seed_tracks.clone(),
-                100,
-                self.country,
-                &Map::new(),
+                seed_trackids.as_ref(),
+                Some(&Market::FromToken),
+                Some(100),
             )
         })
     }
@@ -283,8 +351,17 @@ impl WebApi {
         limit: u32,
         offset: u32,
     ) -> Option<SearchResult> {
-        self.api_with_retry(|api| api.search(query, searchtype, limit, offset, self.country, None))
-            .take()
+        self.api_with_retry(|api| {
+            api.search(
+                query,
+                &searchtype,
+                Some(&Market::FromToken),
+                None,
+                Some(limit),
+                Some(offset),
+            )
+        })
+        .take()
     }
 
     pub fn current_user_playlist(&self) -> ApiResult<Playlist> {
@@ -292,19 +369,21 @@ impl WebApi {
         let spotify = self.clone();
         let fetch_page = move |offset: u32| {
             debug!("fetching user playlists, offset: {}", offset);
-            spotify.api_with_retry(|api| match api.current_user_playlists(MAX_LIMIT, offset) {
-                Ok(page) => Ok(ApiPage {
-                    offset: page.offset,
-                    total: page.total,
-                    items: page.items.iter().map(|sp| sp.into()).collect(),
-                }),
-                Err(e) => Err(e),
+            spotify.api_with_retry(|api| {
+                match api.current_user_playlists_manual(Some(MAX_LIMIT), Some(offset)) {
+                    Ok(page) => Ok(ApiPage {
+                        offset: page.offset,
+                        total: page.total,
+                        items: page.items.iter().map(|sp| sp.into()).collect(),
+                    }),
+                    Err(e) => Err(e),
+                }
             })
         };
         ApiResult::new(MAX_LIMIT, Arc::new(fetch_page))
     }
 
-    pub fn user_playlist_tracks(&self, playlist_id: &str) -> ApiResult<Track> {
+    pub fn user_playlist_tracks(&self, playlist_id: &str) -> ApiResult<Playable> {
         const MAX_LIMIT: u32 = 100;
         let spotify = self.clone();
         let playlist_id = playlist_id.to_string();
@@ -314,13 +393,12 @@ impl WebApi {
                 playlist_id, offset
             );
             spotify.api_with_retry(|api| {
-                match api.user_playlist_tracks(
-                    spotify.user.as_ref().unwrap(),
-                    &playlist_id,
+                match api.playlist_items_manual(
+                    &PlaylistId::from_id(&playlist_id).unwrap(),
                     None,
-                    MAX_LIMIT,
-                    offset,
-                    spotify.country,
+                    Some(&Market::FromToken),
+                    Some(MAX_LIMIT),
+                    Some(offset),
                 ) {
                     Ok(page) => Ok(ApiPage {
                         offset: page.offset,
@@ -331,10 +409,11 @@ impl WebApi {
                             .enumerate()
                             .flat_map(|(index, pt)| {
                                 pt.track.as_ref().map(|t| {
-                                    let mut track: Track = t.into();
-                                    track.added_at = Some(pt.added_at);
-                                    track.list_index = page.offset as usize + index;
-                                    track
+                                    let mut playable: Playable = t.into();
+                                    // TODO: set these
+                                    playable.set_added_at(pt.added_at);
+                                    playable.set_list_index(page.offset as usize + index);
+                                    playable
                                 })
                             })
                             .collect(),
@@ -347,7 +426,7 @@ impl WebApi {
     }
 
     pub fn full_album(&self, album_id: &str) -> Option<FullAlbum> {
-        self.api_with_retry(|api| api.album(album_id))
+        self.api_with_retry(|api| api.album(&AlbumId::from_id(album_id).unwrap()))
     }
 
     pub fn album_tracks(
@@ -356,7 +435,13 @@ impl WebApi {
         limit: u32,
         offset: u32,
     ) -> Option<Page<SimplifiedTrack>> {
-        self.api_with_retry(|api| api.album_track(album_id, limit, offset))
+        self.api_with_retry(|api| {
+            api.album_track_manual(
+                &AlbumId::from_id(album_id).unwrap(),
+                Some(limit),
+                Some(offset),
+            )
+        })
     }
 
     pub fn artist_albums(
@@ -370,10 +455,10 @@ impl WebApi {
         let fetch_page = move |offset: u32| {
             debug!("fetching artist {} albums, offset: {}", artist_id, offset);
             spotify.api_with_retry(|api| {
-                match api.artist_albums(
-                    &artist_id,
-                    album_type,
-                    spotify.country,
+                match api.artist_albums_manual(
+                    &ArtistId::from_id(&artist_id).unwrap(),
+                    album_type.as_ref(),
+                    Some(&Market::FromToken),
                     Some(MAX_SIZE),
                     Some(offset),
                 ) {
@@ -402,7 +487,12 @@ impl WebApi {
         let fetch_page = move |offset: u32| {
             debug!("fetching show {} episodes, offset: {}", &show_id, offset);
             spotify.api_with_retry(|api| {
-                match api.get_shows_episodes(show_id.clone(), MAX_SIZE, offset, spotify.country) {
+                match api.get_shows_episodes_manual(
+                    &ShowId::from_id(&show_id).unwrap(),
+                    Some(&Market::FromToken),
+                    Some(50),
+                    Some(offset),
+                ) {
                     Ok(page) => Ok(ApiPage {
                         offset: page.offset,
                         total: page.total,
@@ -417,71 +507,125 @@ impl WebApi {
     }
 
     pub fn get_saved_shows(&self, offset: u32) -> Option<Page<Show>> {
-        self.api_with_retry(|api| api.get_saved_show(50, offset))
+        self.api_with_retry(|api| api.get_saved_show_manual(Some(50), Some(offset)))
     }
 
-    pub fn save_shows(&self, ids: Vec<String>) -> bool {
-        self.api_with_retry(|api| api.save_shows(ids.clone()))
-            .is_some()
+    pub fn save_shows(&self, ids: Vec<&str>) -> bool {
+        self.api_with_retry(|api| {
+            api.save_shows(
+                &ids.iter()
+                    .map(|id| ShowId::from_id(id).unwrap())
+                    .collect::<Vec<ShowId>>(),
+            )
+        })
+        .is_some()
     }
 
-    pub fn unsave_shows(&self, ids: Vec<String>) -> bool {
-        self.api_with_retry(|api| api.remove_users_saved_shows(ids.clone(), self.country))
-            .is_some()
+    pub fn unsave_shows(&self, ids: Vec<&str>) -> bool {
+        self.api_with_retry(|api| {
+            api.remove_users_saved_shows(
+                &ids.iter()
+                    .map(|id| ShowId::from_id(id).unwrap())
+                    .collect::<Vec<ShowId>>(),
+                Some(&Market::FromToken),
+            )
+        })
+        .is_some()
     }
 
     pub fn current_user_followed_artists(
         &self,
-        last: Option<String>,
+        last: Option<&str>,
     ) -> Option<CursorBasedPage<FullArtist>> {
-        self.api_with_retry(|api| api.current_user_followed_artists(50, last.clone()))
-            .map(|cp| cp.artists)
+        self.api_with_retry(|api| api.current_user_followed_artists(last, Some(50)))
     }
 
-    pub fn user_follow_artists(&self, ids: Vec<String>) -> Option<()> {
-        self.api_with_retry(|api| api.user_follow_artists(&ids))
+    pub fn user_follow_artists(&self, ids: Vec<&str>) -> Option<()> {
+        self.api_with_retry(|api| {
+            api.user_follow_artists(
+                &ids.iter()
+                    .map(|id| ArtistId::from_id(id).unwrap())
+                    .collect::<Vec<ArtistId>>(),
+            )
+        })
     }
 
-    pub fn user_unfollow_artists(&self, ids: Vec<String>) -> Option<()> {
-        self.api_with_retry(|api| api.user_unfollow_artists(&ids))
+    pub fn user_unfollow_artists(&self, ids: Vec<&str>) -> Option<()> {
+        self.api_with_retry(|api| {
+            api.user_unfollow_artists(
+                &ids.iter()
+                    .map(|id| ArtistId::from_id(id).unwrap())
+                    .collect::<Vec<ArtistId>>(),
+            )
+        })
     }
 
     pub fn current_user_saved_albums(&self, offset: u32) -> Option<Page<SavedAlbum>> {
-        self.api_with_retry(|api| api.current_user_saved_albums(50, offset))
+        self.api_with_retry(|api| {
+            api.current_user_saved_albums_manual(Some(&Market::FromToken), Some(50), Some(offset))
+        })
     }
 
-    pub fn current_user_saved_albums_add(&self, ids: Vec<String>) -> Option<()> {
-        self.api_with_retry(|api| api.current_user_saved_albums_add(&ids))
+    pub fn current_user_saved_albums_add(&self, ids: Vec<&str>) -> Option<()> {
+        self.api_with_retry(|api| {
+            api.current_user_saved_albums_add(
+                &ids.iter()
+                    .map(|id| AlbumId::from_id(id).unwrap())
+                    .collect::<Vec<AlbumId>>(),
+            )
+        })
     }
 
-    pub fn current_user_saved_albums_delete(&self, ids: Vec<String>) -> Option<()> {
-        self.api_with_retry(|api| api.current_user_saved_albums_delete(&ids))
+    pub fn current_user_saved_albums_delete(&self, ids: Vec<&str>) -> Option<()> {
+        self.api_with_retry(|api| {
+            api.current_user_saved_albums_delete(
+                &ids.iter()
+                    .map(|id| AlbumId::from_id(id).unwrap())
+                    .collect::<Vec<AlbumId>>(),
+            )
+        })
     }
 
     pub fn current_user_saved_tracks(&self, offset: u32) -> Option<Page<SavedTrack>> {
-        self.api_with_retry(|api| api.current_user_saved_tracks(50, offset))
+        self.api_with_retry(|api| {
+            api.current_user_saved_tracks_manual(Some(&Market::FromToken), Some(50), Some(offset))
+        })
     }
 
-    pub fn current_user_saved_tracks_add(&self, ids: Vec<String>) -> Option<()> {
-        self.api_with_retry(|api| api.current_user_saved_tracks_add(&ids))
+    pub fn current_user_saved_tracks_add(&self, ids: Vec<&str>) -> Option<()> {
+        self.api_with_retry(|api| {
+            api.current_user_saved_tracks_add(
+                &ids.iter()
+                    .map(|id| TrackId::from_id(id).unwrap())
+                    .collect::<Vec<TrackId>>(),
+            )
+        })
     }
 
-    pub fn current_user_saved_tracks_delete(&self, ids: Vec<String>) -> Option<()> {
-        self.api_with_retry(|api| api.current_user_saved_tracks_delete(&ids))
+    pub fn current_user_saved_tracks_delete(&self, ids: Vec<&str>) -> Option<()> {
+        self.api_with_retry(|api| {
+            api.current_user_saved_tracks_delete(
+                &ids.iter()
+                    .map(|id| TrackId::from_id(id).unwrap())
+                    .collect::<Vec<TrackId>>(),
+            )
+        })
     }
 
-    pub fn user_playlist_follow_playlist(&self, owner_id: String, id: String) -> Option<()> {
-        self.api_with_retry(|api| api.user_playlist_follow_playlist(&owner_id, &id, true))
+    pub fn user_playlist_follow_playlist(&self, id: &str) -> Option<()> {
+        self.api_with_retry(|api| api.playlist_follow(&PlaylistId::from_id(id).unwrap(), None))
     }
 
     pub fn artist_top_tracks(&self, id: &str) -> Option<Vec<Track>> {
-        self.api_with_retry(|api| api.artist_top_tracks(id, self.country))
-            .map(|ft| ft.tracks.iter().map(|t| t.into()).collect())
+        self.api_with_retry(|api| {
+            api.artist_top_tracks(&ArtistId::from_id(id).unwrap(), &Market::FromToken)
+        })
+        .map(|ft| ft.iter().map(|t| t.into()).collect())
     }
 
-    pub fn artist_related_artists(&self, id: String) -> Option<Vec<Artist>> {
-        self.api_with_retry(|api| api.artist_related_artists(&id))
-            .map(|fa| fa.artists.iter().map(|a| a.into()).collect())
+    pub fn artist_related_artists(&self, id: &str) -> Option<Vec<Artist>> {
+        self.api_with_retry(|api| api.artist_related_artists(&ArtistId::from_id(id).unwrap()))
+            .map(|fa| fa.iter().map(|a| a.into()).collect())
     }
 
     pub fn current_user(&self) -> Option<PrivateUser> {
