@@ -1,16 +1,16 @@
-use crate::album::Album;
-use crate::artist::Artist;
 use crate::command::Command;
 use crate::commands::CommandResult;
-use crate::episode::Episode;
 use crate::events::EventManager;
 use crate::library::Library;
-use crate::playlist::Playlist;
+use crate::model::album::Album;
+use crate::model::artist::Artist;
+use crate::model::episode::Episode;
+use crate::model::playlist::Playlist;
+use crate::model::show::Show;
+use crate::model::track::Track;
 use crate::queue::Queue;
-use crate::show::Show;
 use crate::spotify::{Spotify, UriType};
 use crate::spotify_url::SpotifyUrl;
-use crate::track::Track;
 use crate::traits::{ListItem, ViewExt};
 use crate::ui::listview::ListView;
 use crate::ui::pagination::Pagination;
@@ -18,7 +18,7 @@ use crate::ui::tabview::TabView;
 use cursive::view::ViewWrapper;
 use cursive::Cursive;
 use rspotify::model::search::SearchResult;
-use rspotify::senum::SearchType;
+use rspotify::model::SearchType;
 use std::sync::{Arc, RwLock};
 
 pub struct SearchResultsView {
